package com.akamai.osmf.samples
{
    import com.akamai.osmf.samples.presenter.BaseControlBarPresenter;
    import com.akamai.osmf.samples.view.BaseVideoView;
    
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.SecurityErrorEvent;
    import flash.net.URLLoader;
    import flash.net.URLRequest;
    import flash.utils.getDefinitionByName;
    
    import org.osmf.events.MediaFactoryEvent;
    import org.osmf.media.MediaResourceBase;
    import org.osmf.media.PluginInfoResource;
    import org.osmf.media.URLResource;
    import org.osmf.utils.OSMFSettings;
    
    public class BaseSample extends Sprite
    {
		// Variables with getters/setters
        private var _view:BaseVideoView;
		// Variables with no getters/setters
        private var controlBarPresenter:BaseControlBarPresenter;

        public function BaseSample()
        {
            super();
            
            addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
        }
		        
        public function get view():BaseVideoView
        {
            return _view;
        }
        
        protected function onAddedToStage(event:Event):void
        {
            removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			addView();
            setStageProperties();
			loadPlugin(pluginURL);
        }
		
		/**
		 * Override in samples to specify the plugin URL.
		 **/
		protected function get pluginURL():String
		{
			return "";
		}
		
        protected function setStageProperties():void
        {
            stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;
         	
			// remove the player background if the system supports stage video
			if (OSMFSettings.supportsStageVideo)
			{
				_view.background.visible = false;
			}
        }
		
		protected function addView():void
        {
            _view = createView();
            addChild(_view);
        }
        
        /**
         * Override to provide your custom view.
         **/
        protected function createView():BaseVideoView
        {
            return new BaseVideoView();
        }
        
        /**
         * Override to provide your control bar presenter.
         **/
        protected function createControlBarPresenter():BaseControlBarPresenter
        {
            return null;
        }
        
        protected function loadPlugin(source:String):void
        {
            var pluginResource:MediaResourceBase;
            
            if (source.search(/^http|^file/i) == -1)
            {
                // This is a class
                var pluginInfoRef:Class = flash.utils.getDefinitionByName(source) as Class;
                pluginResource = new PluginInfoResource(new pluginInfoRef);
            }
            else
            {
                // This is a URL
                pluginResource = new URLResource(source);
            }
            
            setupListeners();
            view.mediaPlayerSprite.mediaFactory.loadPlugin(pluginResource);
            
            function setupListeners(add:Boolean = true):void
            {
                if (add)
                {
                    view.mediaPlayerSprite.mediaFactory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD, onPluginLoad);
                    view.mediaPlayerSprite.mediaFactory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD_ERROR, onPluginLoadError);
                }
                else
                {
                    view.mediaPlayerSprite.mediaFactory.removeEventListener(MediaFactoryEvent.PLUGIN_LOAD, onPluginLoad);
                    view.mediaPlayerSprite.mediaFactory.removeEventListener(MediaFactoryEvent.PLUGIN_LOAD_ERROR, onPluginLoadError);
                }
            }
            
            function onPluginLoad(event:MediaFactoryEvent):void
            {
                debug("plugin loaded successfully: " + pluginResource);
                setupListeners(false);
                
                controlBarPresenter = createControlBarPresenter();
            }
            
            function onPluginLoadError(event:MediaFactoryEvent):void
            {
                debug("plugin failed to load: " + pluginURL);
                setupListeners(false);
            }
        }
    
		private function debug(msg:String):void
		{
			trace(msg);
		}
    }
}
